<form action="" method="post">
    {% csrf_token %}
    <p>Are you sure you want to delete "{{ object }}"?</p>
    <div class="form-group">
        <div class="d-flex justify-content-center">
            <input type="submit" value="Confirm" class="btn btn-outline-danger"/>
        </div>
    </div>
</form>