{% load static %}
<script src="{% static 'process/js/pr_highcharts.js' %}"></script>
<script src="{% static 'process/js/pr_sankey.js' %}"></script>
<script src="{% static 'process/js/pr_organization.js' %}"></script>
<script src="{% static 'process/js/pr_exporting.js' %}"></script>
<link rel="stylesheet" type="text/css" href="{% static 'process/css/pr_highcharts.css' %}">
<link rel="stylesheet" type="text/css" href="{% static 'process/css/pr_organization.css' %}">